@@ -10,6 +10,7 @@ from typing import Tuple
 from util_hj3415 import utils
 
 import logging
+
 logger = logging.getLogger(__name__)
 formatter = logging.Formatter('%(levelname)s: [%(name)s] %(message)s')
 ch = logging.StreamHandler()
@@ -169,7 +170,8 @@ class MI(Atlas):
         """저장된 가장 최근의 값을 반환하는 함수
         """
         if self.chk_col():
-            d = self.client[self.db][self.index].find({'date': {'$exists': True}}).sort('date', pymongo.DESCENDING).next()
+            d = self.client[self.db][self.index].find({'date': {'$exists': True}}).sort('date',
+                                                                                        pymongo.DESCENDING).next()
             del d['_id']
             return d['date'], d['value']
 
@@ -185,13 +187,10 @@ class MI(Atlas):
         if self.index is None:
             raise Exception("You should set index first.")
         self.client[self.db][self.index].create_index('date', unique=True)
-        try:
-            result = self.client[self.db][self.index].insert_one(mi_dict)
-        except errors.DuplicateKeyError:
-            self.client[self.db][self.index].delete_many({'date': {"$eq": mi_dict['date']}})
-            result = self.client[self.db][self.index].insert_one(mi_dict)
+        result = self.client[self.db][self.index].update_one(
+            {'date': mi_dict['date']}, {"$set": {'value': mi_dict['value']}}, upsert=True)
         return result.acknowledged
-    
+
     def drop_db(self, db: str = 'mi'):
         super(MI, self).drop_db(db)
 
@@ -203,6 +202,7 @@ class NotiHx(Atlas):
 
     """
     NOTI_DB = 'noti'
+
     # COL의 형식은 %Y%m
 
     def __init__(self, client):
@@ -395,20 +395,11 @@ class Corps(Atlas):
             self.drop_db(corp_db)
 
     def _save_df(self, df: pd.DataFrame) -> bool:
-        # c106에서 주로 사용하는 저장방식
+        # c103, c104, c106, c108에서 주로 사용하는 저장방식
         if df.empty:
             print('Dataframe is empty..So we will skip saving db..')
             return False
-        self.client[self.code][self.page].insert_one({'test':'test','e':'d'})
-        self.clear_col()
-        result = self.client[self.code][self.page].insert_many(df.to_dict('index'))
-        return result.acknowledged
-
-    def _save_list(self, list_data: list) -> bool:
-        # 주로 c108, c103, c104에서 사용하는 저장방식
-        # delete all documents in a collection
-        self.clear_col()
-        result = self.client[self.code][self.page].insert_many(list_data)
+        result = self.client[self.code][self.page].insert_many(df.to_dict('records'))
         return result.acknowledged
 
     def _save_dict(self, dict_data: dict, del_query: dict) -> bool:
@@ -522,7 +513,8 @@ class C101(Corps):
             'intro': '한국 및 CE... DP사업으로 구성됨.'}\n
         """
         try:
-            d = self.client[self.code][self.page].find({'date': {'$exists': True}}).sort('date', pymongo.DESCENDING).next()
+            d = self.client[self.code][self.page].find({'date': {'$exists': True}}).sort('date',
+                                                                                         pymongo.DESCENDING).next()
             del d['_id']
         except StopIteration:
             d = {}
@@ -532,6 +524,7 @@ class C101(Corps):
 class CDart(Corps):
     """각 Code 별로 분류된 Dart 를 관리하는 클래스
     """
+
     def __init__(self, client, code: str):
         super(CDart, self).__init__(client)
         self.code = code
@@ -572,8 +565,6 @@ class CDart(Corps):
             logger.error(f'Error occurred while delete dart data..')
 
 
-# ====== 테스트 필요
-
 class C106Y(Corps):
     def __init__(self, client, code: str):
         """
@@ -587,6 +578,8 @@ class C106Y(Corps):
         self.page = 'c106y'
 
     def save(self, c106_data: pd.DataFrame) -> bool:
+        self.client[self.code][self.page].create_index('항목', unique=True)
+        self.clear_col()
         return super(C106Y, self)._save_df(c106_data)
 
     def load(self) -> pd.DataFrame:
@@ -606,6 +599,8 @@ class C106Q(Corps):
         self.page = 'c106q'
 
     def save(self, c106_data: pd.DataFrame) -> bool:
+        self.client[self.code][self.page].create_index('항목', unique=True)
+        self.clear_col()
         return super(C106Q, self)._save_df(c106_data)
 
     def load(self) -> pd.DataFrame:
@@ -618,19 +613,10 @@ class C108(Corps):
         self.code = code
         self.page = 'c108'
 
-    def save(self, c108_data: list) -> bool:
-        """
-
-        Args:
-            c108_data (list): c108로 저장할 리스트 형식
-        Note:
-            c108_list 구조\n
-            [{'날짜': '21/07/19', '제목': '...',\n
-            '작성자': '...', '제공처': '...', \n
-            '투자의견': '...', '목표가': '95,000',\n
-            '내용': '...'},]\n
-        """
-        return super(C108, self)._save_list(c108_data)
+    def save(self, c108_data: pd.DataFrame) -> bool:
+        self.client[self.code][self.page].insert_one({})
+        self.clear_col()
+        return super(C108, self)._save_df(c108_data)
 
     def get_all(self) -> list:
         """
@@ -653,7 +639,9 @@ class C108(Corps):
         """
         # 저장되어 있는 데이터베이스의 최근 날짜를 찾는다.
         try:
-            r_date = self.client[self.code][self.page].find({'날짜': {'$exists': True}}).sort('날짜', pymongo.DESCENDING).next()['날짜']
+            r_date = \
+            self.client[self.code][self.page].find({'날짜': {'$exists': True}}).sort('날짜', pymongo.DESCENDING).next()[
+                '날짜']
         except StopIteration:
             # 날짜에 해당하는 데이터가 없는 경우
             return []
@@ -666,13 +654,109 @@ class C108(Corps):
         return r_list
 
 
+class C1034(Corps):
+    pass
+
+
+class C103(C1034):
+    """C103 컬렉션 관련 클래스
+    """
+
+    def __init__(self, client, code: str, page: str):
+        """
+
+        Args:
+            code (str): 종목코드(디비명)
+            page (str): 페이지명(컬렉션명)
+
+        Example:
+            c103손익계산서q\n
+            c103재무상태표y\n
+            c103현금흐름표q\n
+
+        """
+        super(C103, self).__init__(client)
+        self.code = code
+        self.page = page
+
+    def save(self, c103_data: pd.DataFrame) -> bool:
+        """데이터베이스에 저장
+
+        Example:
+            c103_list 예시\n
+            [{'항목': '자산총계', '2020/03': 3574575.4, ... '전분기대비': 3.9},
+            {'항목': '유동자산', '2020/03': 1867397.5, ... '전분기대비': 5.5}]
+
+        Note:
+            항목이 중복되는 경우가 있기 때문에 c104처럼 각 항목을 키로하는 딕셔너리로 만들지 않는다.
+        """
+        self.client[self.code][self.page].create_index('항목', unique=False)
+        self.clear_col()
+        return super(C103, self)._save_df(c103_data)
+
+
+class C104(C1034):
+    """C104 컬렉션 관련 클래스
+    """
+
+    def __init__(self, client, code: str, page: str):
+        """
+
+        Args:
+            code (str): 종목코드(디비명)
+            page (str): c104y, c104q(컬렉션명)
+        """
+        super(C104, self).__init__(client)
+        self.code = code
+        self.page = page
+
+    def save(self, c104_data: pd.DataFrame) -> bool:
+        """데이터베이스에 저장
+
+        c104는 4페이지의 자료를 한 컬렉션에 모으는 것이기 때문에
+        stamp 를 검사하여 12시간 전보다 이전에 저장된 자료가 있으면
+        삭제한 후 저장하고 12시간 이내의 자료는 삭제하지 않고
+        데이터를 추가하는 형식으로 저장한다.
+
+        Example:
+            c104_data 예시\n
+            [{'항목': '매출액증가율',...'2020/12': 2.78, '2021/12': 14.9, '전년대비': 8.27, '전년대비1': 12.12},
+            {'항목': '영업이익증가율',...'2020/12': 29.62, '2021/12': 43.86, '전년대비': 82.47, '전년대비1': 14.24}]
+
+        Note:
+            항목이 중복되는 경우가 있기 때문에 c104처럼 각 항목을 키로하는 딕셔너리로 만들지 않는다.
+        """
+        self.client[self.code][self.page].create_index('항목', unique=True)
+        time_now = datetime.datetime.now()
+        try:
+            stamp = self.client[self.code][self.page].find_one({'항목': 'stamp'})['time']
+            if stamp < (time_now - datetime.timedelta(days=.5)):
+                # 스템프가 12시간 이전이라면..연속데이터가 아니라는 뜻이므로 컬렉션을 초기화한다.
+                self.clear_col()
+        except TypeError:
+            # 스템프가 없다면...
+            pass
+        # 항목 stamp를 찾아 time을 업데이트하고 stamp가 없으면 insert한다.
+        self.client[self.code][self.page].update_one({'항목': 'stamp'}, {"$set": {'time': time_now}}, upsert=True)
+        return super(C104, self)._save_df(c104_data)
+
+    def modify_stamp(self, days_ago: int):
+        # 인위적으로 타임스템프를 수정한다 - 테스트 용도
+        try:
+            before = self.client[self.code][self.page].find_one({'항목': 'stamp'})['time']
+        except TypeError:
+            # 이전에 타임스템프가 없는 경우
+            before = None
+        time_2da = datetime.datetime.now() - datetime.timedelta(days=days_ago)
+        self.client[self.code][self.page].update_one({'항목': 'stamp'}, {"$set": {'time': time_2da}}, upsert=True)
+        after = self.client[self.code][self.page].find_one({'항목': 'stamp'})['time']
+        print(f"Stamp changed: {before} -> {after}")
 
 
 # ================================= 준비중 =====================================
 
 
-
-class C1034(Corps):
+class C1034_Old(Corps):
 
     def get_all_title(self) -> list:
         """
@@ -792,7 +876,7 @@ class C1034(Corps):
 
     # ========================특정 페이지 관련 함수들=======================
 
-    @ staticmethod
+    @staticmethod
     def _nan_to_zero(target: Tuple[str, float]) -> Tuple[str, float]:
         """
 
@@ -870,7 +954,7 @@ class C1034(Corps):
             logger.info(f'{title} : {od_q}')
             if len(od_q) < 4:
                 # od_q의 값이 4개 이하이면 그냥 최근 연도의 값으로 반환한다.
-                t = C1034(code=self.db, page=self.col[:-1]+'y').latest_value(title=title)
+                t = C1034(code=self.db, page=self.col[:-1] + 'y').latest_value(title=title)
                 if nan_to_zero:
                     return self._nan_to_zero(t)
                 else:
@@ -941,139 +1025,9 @@ class C1034(Corps):
             return tuple(last_one)
 
 
-class C104(C1034):
-    """C104 컬렉션 관련 클래스
-    """
-    def __init__(self, code: str, page: str):
-        """
-
-        Args:
-            code (str): 종목코드(디비명)
-            page (str): c104y, c104q(컬렉션명)
-        """
-        super(C104, self).__init__()
-        self.code = code
-        self.page = page
-
-    def save(self, c104_list: list) -> bool:
-        """데이터베이스에 저장
-
-        c104는 4페이지의 자료를 한 컬렉션에 모으는 것이기 때문에
-        stamp 를 검사하여 12시간 전보다 이전에 저장된 자료가 있으면
-        삭제한 후 저장하고 12시간 이내의 자료는 삭제하지 않고
-        데이터를 추가하는 형식으로 저장한다.
-
-        Args:
-            c104_list (list): 딕셔너리를 담은 리스트 형식
-
-        Returns:
-            bool: 저장 성공 실패 여부
-
-        Example:
-            c104_list 예시\n
-            [{'항목': '매출액증가율',...'2020/12': 2.78, '2021/12': 14.9, '전년대비': 8.27, '전년대비1': 12.12},
-            {'항목': '영업이익증가율',...'2020/12': 29.62, '2021/12': 43.86, '전년대비': 82.47, '전년대비1': 14.24}]
-
-        Note:
-            항목이 중복되는 경우가 있기 때문에 c104처럼 각 항목을 키로하는 딕셔너리로 만들지 않는다.
-        """
-        time_now = datetime.datetime.now()
-        # c104가 리스트라서 append 사용
-        c104_list.append({'stamp': time_now})
-        if self.client[self.code][self.page].find_one({'stamp': {'$lt': time_now - datetime.timedelta(days=.5)}}):
-            # delete all documents in a collection
-            self.clear_col()
-        result = self.client[self.code][self.page].insert_many(c104_list)
-        return result.acknowledged
-
-
-class C103(C1034):
-    """C103 컬렉션 관련 클래스
-    """
-    def __init__(self, code: str, page: str):
-        """
-
-        Args:
-            code (str): 종목코드(디비명)
-            page (str): 페이지명(컬렉션명)
-
-        Example:
-            c103손익계산서q (c103손q, c103iq, c103_iq)\n
-            c103재무상태표y (c103재q, c103bq, c103_bq)\n
-            c103현금흐름표q (c103현q, c103cq, c103_cq)\n
-
-        """
-        super(C103, self).__init__()
-        self.code = code
-        self.page = page
-
-    def save(self, c103_list: list) -> bool:
-        """데이터베이스에 저장
-
-        Args:
-            c103_list (list): 딕셔너리를 담은 리스트 형식
-
-        Returns:
-            bool: 저장 성공 실패 여부
-
-        Example:
-            c103_list 예시\n
-            [{'항목': '자산총계', '2020/03': 3574575.4, ... '전분기대비': 3.9},
-            {'항목': '유동자산', '2020/03': 1867397.5, ... '전분기대비': 5.5}]
-
-        Note:
-            항목이 중복되는 경우가 있기 때문에 c104처럼 각 항목을 키로하는 딕셔너리로 만들지 않는다.
-        """
-        # delete all documents in a collection
-        self.clear_col()
-        result = self.client[self.db][self.col].insert_many(c103_list)
-        return result.acknowledged
-
-
-
-
-
-
-
-
 if __name__ == "__main__":
     atlas_addr = "mongodb+srv://hj3415:piyrw421@nfs.2mnb4.mongodb.net/005930?retryWrites=true&w=majority"
     local_addr = 'mongodb://localhost:27017'
 
     client = connect_mongo(atlas_addr)
-    #Atlas(client)
-    data = {'전일종가': {'린드먼아시아': '6500', '삼성스팩2호': '8280', '큐캐피탈': '4085', '엠벤처투자': '305', '나우IB': '14100'},
-            '시가총액': {'린드먼아시아': '877.5', '삼성스팩2호': '854.1', '큐캐피탈': '885.5', '엠벤처투자': '835.0', '나우IB': '890.3'},
-            '재무상태표': {'린드먼아시아': float('nan'), '삼성스팩2호': float('nan'), '큐캐피탈': float('nan'), '엠벤처투자': float('nan'),
-                      '나우IB': float('nan')},
-            '자산총계': {'린드먼아시아': '655.6', '삼성스팩2호': '545.1', '큐캐피탈': '1430.3', '엠벤처투자': '1250.3', '나우IB': '681.3'},
-            '부채총계': {'린드먼아시아': '167.1', '삼성스팩2호': '225.1', '큐캐피탈': '653.8', '엠벤처투자': '492.4', '나우IB': '198.9'},
-            '포괄손익계산서': {'린드먼아시아': float('nan'), '삼성스팩2호': float('nan'), '큐캐피탈': float('nan'), '엠벤처투자': float('nan'),
-                        '나우IB': float('nan')},
-            '매출액': {'린드먼아시아': '115.5', '삼성스팩2호': '446.1', '큐캐피탈': '643.0', '엠벤처투자': '280.4', '나우IB': '435.5'},
-            '영업이익': {'린드먼아시아': '65.5', '삼성스팩2호': '53.2', '큐캐피탈': '-69.5', '엠벤처투자': '-86.3', '나우IB': '58.5'},
-            '당기순이익': {'린드먼아시아': '50.8', '삼성스팩2호': '52.0', '큐캐피탈': '-39.3', '엠벤처투자': '-165.2', '나우IB': '44.2'},
-            '당기순이익(지배)': {'린드먼아시아': '50.8', '삼성스팩2호': '52.0', '큐캐피탈': '-39.3', '엠벤처투자': '-128.9', '나우IB': '44.2'},
-            '수익성지표': {'린드먼아시아': float('nan'), '삼성스팩2호': float('nan'), '큐캐피탈': float('nan'), '엠벤처투자': float('nan'),
-                      '나우IB': float('nan')},
-            '영업이익률': {'린드먼아시아': '56.68', '삼성스팩2호': '11.92', '큐캐피탈': '-10.81', '엠벤처투자': '-30.78', '나우IB': '13.44'},
-            '순이익률': {'린드먼아시아': '43.96', '삼성스팩2호': '11.67', '큐캐피탈': '-6.12', '엠벤처투자': '-58.94', '나우IB': '10.16'},
-            'ROE': {'린드먼아시아': '10.79', '삼성스팩2호': '17.73', '큐캐피탈': '-5.62', '엠벤처투자': '-18.00', '나우IB': '9.63'},
-            '안정성지표': {'린드먼아시아': float('nan'), '삼성스팩2호': float('nan'), '큐캐피탈': float('nan'), '엠벤처투자': float('nan'),
-                      '나우IB': float('nan')},
-            '부채비율': {'린드먼아시아': '34.22', '삼성스팩2호': '70.36', '큐캐피탈': '84.21', '엠벤처투자': '64.96', '나우IB': '41.23'},
-            '가치지표': {'린드먼아시아': float('nan'), '삼성스팩2호': float('nan'), '큐캐피탈': float('nan'), '엠벤처투자': float('nan'),
-                     '나우IB': float('nan')},
-            'PER': {'린드먼아시아': '13.96', '삼성스팩2호': '25.67', '큐캐피탈': float('nan'), '엠벤처투자': float('nan'),
-                    '나우IB': '21.41'},
-            'PBR': {'린드먼아시아': '1.43', '삼성스팩2호': '4.06', '큐캐피탈': '1.19', '엠벤처투자': '1.50', '나우IB': '1.96'},
-            '투자의견': {'린드먼아시아': float('nan'), '삼성스팩2호': float('nan'), '큐캐피탈': float('nan'), '엠벤처투자': float('nan'),
-                     '나우IB': float('nan')},
-            '목표주가': {'린드먼아시아': float('nan'), '삼성스팩2호': float('nan'), '큐캐피탈': float('nan'), '엠벤처투자': float('nan'),
-                     '나우IB': float('nan')},
-            '재무연월': {'린드먼아시아': '2020/12', '삼성스팩2호': '2020/12', '큐캐피탈': '2020/12', '엠벤처투자': '2020/12',
-                     '나우IB': '2020/12'}}
-    print(pd.DataFrame(data).to_dict('index'))
-    #C106Q(client,'005930').save(pd.DataFrame(data))
-
-
+    Atlas(client).initiate_db()
